'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _isBuffer = require('is-buffer');

var _isBuffer2 = _interopRequireDefault(_isBuffer);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('akita:response');

var Response = function () {
  function Response(fetch, path, init, query) {
    _classCallCheck(this, Response);

    this._query = query;
    this._path = path;
    this._init = init;

    var promise = fetch(path, init);
    if (debug.enabled) {
      promise = promise.then(function (res) {
        debug('response status:', res.status, res.statusText);
        return res;
      }, function (error) {
        debug('fetch error:', error.message);
        return Promise.reject(error);
      });
    }
    this._responsePromise = promise;
  }

  _createClass(Response, [{
    key: 'inspect',
    value: function inspect() {
      return Object.assign({}, this._init, { url: this._path });
    }
  }, {
    key: 'response',
    value: function response() {
      return this._responsePromise;
    }
  }, {
    key: 'stream',
    value: function stream() {
      return this.response().then(function (res) {
        return res.body;
      });
    }
  }, {
    key: 'ok',
    value: function ok() {
      return this.response().then(function (res) {
        return res.ok;
      });
    }
  }, {
    key: 'status',
    value: function status() {
      return this.response().then(function (res) {
        return res.status;
      });
    }
  }, {
    key: 'statusText',
    value: function statusText() {
      return this.response().then(function (res) {
        return res.statusText;
      });
    }
  }, {
    key: 'size',
    value: function size() {
      return this.response().then(function (res) {
        return res.size;
      });
    }
  }, {
    key: 'headers',
    value: function headers() {
      return this.response().then(function (res) {
        return res.headers;
      });
    }

    /**
     * 获取Buffer数据
     * @returns {Promise.<any>}
     */

  }, {
    key: 'buffer',
    value: function buffer() {
      return this.response().then(function (res) {
        var fn = 'buffer';
        if (res.arrayBuffer) {
          fn = 'arrayBuffer';
        }
        return res[fn]().then(function (buf) {
          if (!(0, _isBuffer2.default)(buf) && buf instanceof ArrayBuffer && typeof Buffer === 'function') {
            return Buffer.from(buf);
          }
          return buf;
        }, function (error) {
          debug('get buffer error:', error.message);
          return Promise.reject(error);
        });
      });
    }
  }, {
    key: 'blob',
    value: function blob() {
      return this.response().then(function (res) {
        return res.blob();
      });
    }

    /**
     * 获取文本数据
     * @returns {Promise<string>}
     */

  }, {
    key: 'text',
    value: function text() {
      return this.response().then(function (res) {
        return res.text();
      });
    }

    /**
     * 获取JSON数据
     * @returns {Promise.<any>}
     */

  }, {
    key: 'json',
    value: function json() {
      var _this = this;

      return this.response().then(function (res) {
        if (res.status === 204) {
          return null;
        }
        return res.json().then(function (json) {
          debug('response json:', json);
          if (res.status === 404 && _this._query && ['findById', 'remove'].indexOf(_this._query._op) > -1) {
            debug('return null when ' + _this._query._op + ' 404');
            return null;
          }
          if (json && json.error && ['0', 'null', 'none'].indexOf(json.error) < 0) {
            var error = new Error(json.error);
            // $Flow error.code
            error.code = json.code || json.errorCode || 0;
            return Promise.reject(error);
          }
          return json;
        }, function (error) {
          debug('json parse error:', error.message);
          return Promise.reject(error);
        });
      });
    }

    /**
     * 获取JSON数据
     * @returns {Promise<any>}
     */

  }, {
    key: 'then',
    value: function then(onSuccess, onFail) {
      return this.json().then(onSuccess, onFail);
    }
  }, {
    key: 'catch',
    value: function _catch(onFail) {
      return this.json().catch(onFail);
    }
  }]);

  return Response;
}();

exports.default = Response;